/***************************************\
* w9xstart.c                            *
* Changes the boot screen in Microsoft  *
* Windows 95/98/ME                      *
\***************************************/

/* Copyright 2000-2002 Damian Yerrick; all rights reserved.

This program is free software and comes with ABSOLUTELY NO WARRANTY.
You may redistribute it or modify it subject to the terms and conditions
of the GNU General Public License, version 2 or later. To view this
License, point your Web browser at http://www.gnu.org/copyleft/gpl.html
or view the COPYING file.

 Damian Yerrick \n 5500 Wabash Ave. CM 398 \n Terre Haute IN 47803 \n USA
 mailto:w9xstart@pineight.8m.com
   http://www.pineight.com/

You must distribute this docs/source file if you distribute the binary. */


/* CHANGES

2002 Mar 11: updated with new locations of MinGW and Allegro
2001 Aug 4: port to Win32
2000 Nov: original release

*/


/* USER DOCUMENTATION

When Windows 95, Windows 98, or Windows ME starts up, it displays a
splash screen stored in a 256-color bitmap called logo.sys, stored in
the C:\ folder.  w9xstart changes c:\logo.sys to a randomly selected
graphic from a set on your hard disk.

1. Create a folder "screens" in the same folder as w9xstart.exe.

2. Drop bitmaps (.bmp or .pcx only) into this folder.  To get support
   for .png, .tif, or .jpg, install their respective Allegro addons
   and modify this file to register them with the Allegro library.


If you want to change startup screens every time you start Windows:

3. Create a shortcut to w9xstart.exe and put it in
   Start > Programs > Startup

4. Restart your computer.

*/


/* HOW TO BUILD FROM SOURCE

1. Get Windows 95 or Windows 98.

2. Download and install the MinGW compiler
     http://www.mingw.org/
   and the Allegro library.
     http://alleg.sourceforge.net/
   You'll also need to add headers and import libraries for DirectX
   version 7 or later; instructions to do this come with Allegro.

   If you don't have MinGW, but you do have DJ Delorie's DJGPP,
   you can use it instead.
     http://www.delorie.com/djgpp/
   Because DJGPP programs link the Allegro library statically, you
   can save space by running  UPX on it.  UPX strips debug symbols
   from an executable, compresses it, and leaves a short stub at the
   beginning to decompress it at runtime.  UPX works on most DOS,
   Windows, and Linux apps.
     http://upx.tsx.org/

3. cd to the folder that contains this file, and enter the command
   gcc -Wall w9xstart.c -lalleg -o w9xstart.exe
   If you added extra file types, add their respective libraries
   before -lalleg.
   For example, LoadPNG would have -lldpng -lpng -lz -lalleg

*/




/* THE CODE ****************************/

#define USE_CONSOLE

#include <stdio.h>
#include <time.h>
#include <string.h>
#include <allegro.h>

#ifdef DJGPP
/* The only major platform on which Allegro statically links is DJGPP.
   In that case, help ld drop stuff that's not required for
   SYSTEM_NONE operation.
 */
BEGIN_JOYSTICK_DRIVER_LIST
END_JOYSTICK_DRIVER_LIST
BEGIN_MIDI_DRIVER_LIST
END_MIDI_DRIVER_LIST
BEGIN_DIGI_DRIVER_LIST
END_DIGI_DRIVER_LIST
BEGIN_GFX_DRIVER_LIST
END_GFX_DRIVER_LIST
#endif

/* SwitchFile() **********************************
   Copies the contents of the BMP, PCX, or TGA in infile to a 320x400
   8-bit BMP in outfile. If infile is 320x400, it's copied pixel-for-pixel;
   otherwise, it's scaled so as to compensate for mode z's stretching.
   Returns 0 for success; -1 for failure.
 */
int SwitchFile(const char *outfile, const char *infile)
{
  BITMAP *inbmp;
  BITMAP *outbmp = create_bitmap(320, 400);
  PALETTE pal;
  int outwidth = 320, outheight = 400;

  // braindead Allegro wants char* instead of const char* here
  // ignore the warning about dropped qualifiers
  inbmp = load_bitmap(infile, pal);
  if(inbmp == NULL || outbmp == NULL)
    return 2;

  clear(outbmp);
  
  if(inbmp->w != 320 || inbmp->h != 400)
    {
      if(inbmp->w * 3 > inbmp->h * 4) // letterbox or portrait?
	{
	  // handle letterbox
	  outwidth = 320;
	  outheight = 1600 * inbmp->h / (3 * inbmp->w);
	}
      else
	{
	  // handle portrait
	  outheight = 400;
	  outwidth = 240 * inbmp->w / inbmp->h;
	}
	
    }

  select_palette(pal);
  clear_to_color(outbmp, makecol(0, 0, 0));
  
  stretch_blit(inbmp, outbmp,
	       0, 0, inbmp->w, inbmp->h,
	       (320 - outwidth) / 2, (400 - outheight) / 2,
	       outwidth, outheight);
  /*  
  rectfill(outbmp, (320 - outwidth) / 2, (400 - outheight) / 2,
	   319 - (320 - outwidth) / 2, 399 - (400 - outheight) / 2, 255);
  rect(outbmp, (320 - outwidth) / 2 + 1, (400 - outheight) / 2 + 1,
       318 - (320 - outwidth) / 2, 398 - (400 - outheight) / 2, 0);
  */

  destroy_bitmap(inbmp);
  
  // braindead Allegro wants char* instead of const char* here
  // ignore the warning about dropped qualifiers
  save_bmp(outfile, outbmp, pal);
  destroy_bitmap(outbmp);
  return 0;
}


/* CountFilesCB() **********************
 * Increments the int CountFilesCB_no
 * Useful to count files with Allegro's for_each_file()
 * Not thread safe
 */

int CountFilesCB_no;
void CountFilesCB(const char *unused1, int unused2, int unused3)
{
  CountFilesCB_no++;
}


/* GetNthFileCB() **********************
 * Gets the (n + 1)th string passed to it.
 * Useful with Allegro's for_each_file()
 */

char GetNthFileCB_name[256];
void GetNthFileCB(const char *filename, int unused2, int n)
{
  if(n == CountFilesCB_no)
    strncpy(GetNthFileCB_name, filename, 255);
  CountFilesCB(filename, unused2, n);  
}


/* main() *****************************/

int main(int argc, char **argv)
{
  int nFiles, whichFile;

  install_allegro(SYSTEM_NONE, &errno, atexit);

  set_color_depth(8);

  CountFilesCB_no = 0;
  for_each_file("screens/*.bmp", 0, CountFilesCB, 0);
  for_each_file("screens/*.pcx", 0, CountFilesCB, 0);
  nFiles = CountFilesCB_no;

  if(nFiles == 0)
    return 1;

  srand(time(NULL));
  whichFile = rand() % nFiles;

  CountFilesCB_no = 0;
  for_each_file("screens/*.bmp", 0, GetNthFileCB, whichFile);
  for_each_file("screens/*.pcx", 0, GetNthFileCB, whichFile);

  return SwitchFile("C:\\logo.sys", GetNthFileCB_name);
} END_OF_MAIN();


/*

later, Damian Yerrick */

